@@ -17,7 +17,7 @@
 
 #include <libxml/tree.h>
 #include <libxml/xinclude.h>
-#include <libxml/parser.h> // XML_PARSE_NOXINCNODE
+#include <libxml/parser.h> // XML_PARSE_NOXINCNODE, XML_PARSE_NOBASEFIX
 
 #include <iostream>
 #include <map>
@@ -139,7 +139,7 @@ static const char* get_encoding_or_utf8(const Glib::ustring& encoding)
 {
   if(encoding.empty())
   {
-    //If we don't specify this to the xmlDocDump* functions (using 0 instead),
+    //If we don't specify this to the xmlDocDump* functions (using nullptr instead),
     //then some other encoding is used, causing them to fail on non-ASCII characters.
     return "UTF-8";
   }
@@ -202,7 +202,7 @@ Dtd* Document::get_internal_subset() const
 {
   auto dtd = xmlGetIntSubset(impl_);
   if(!dtd)
-    return 0;
+    return nullptr;
 
   if(!dtd->_private)
     dtd->_private = new Dtd(dtd);
@@ -216,8 +216,8 @@ void Document::set_internal_subset(const Glib::ustring& name,
 {
   auto dtd = xmlCreateIntSubset(impl_,
 				   (const xmlChar*)name.c_str(),
-				   external_id.empty() ? (const xmlChar*)0 : (const xmlChar*)external_id.c_str(),
-				   system_id.empty() ? (const xmlChar*)0 : (const xmlChar*)system_id.c_str());
+				   external_id.empty() ? nullptr : (const xmlChar*)external_id.c_str(),
+				   system_id.empty() ? nullptr : (const xmlChar*)system_id.c_str());
 
   if (dtd && !dtd->_private)
     dtd->_private = new Dtd(dtd);
@@ -227,7 +227,7 @@ Element* Document::get_root_node() const
 {
   auto root = xmlDocGetRootElement(impl_);
   if(root == nullptr)
-    return 0;
+    return nullptr;
   else
   {
     Node::create_wrapper(root);
@@ -239,7 +239,7 @@ Element* Document::create_root_node(const Glib::ustring& name,
                                     const Glib::ustring& ns_uri,
                                     const Glib::ustring& ns_prefix)
 {
-  auto node = xmlNewDocNode(impl_, 0, (const xmlChar*)name.c_str(), 0);
+  auto node = xmlNewDocNode(impl_, nullptr, (const xmlChar*)name.c_str(), nullptr);
   if (!node)
     throw internal_error("Could not create root element node " + name);
 
@@ -266,7 +266,7 @@ Element* Document::create_root_node_by_import(const Node* node,
 					      bool recursive)
 {
   if (!node)
-    return 0;
+    return nullptr;
 
   //Create the node, by copying:
   auto imported_node = xmlDocCopyNode(const_cast<xmlNode*>(node->cobj()), impl_, recursive);
@@ -412,14 +412,19 @@ void Document::set_entity_declaration(const Glib::ustring& name, XmlEntityType t
                               const Glib::ustring& content)
 {
   auto entity = xmlAddDocEntity( impl_, (const xmlChar*) name.c_str(), type,
-    publicId.empty() ? (const xmlChar*)0 : (const xmlChar*)publicId.c_str(),
-    systemId.empty() ? (const xmlChar*)0 : (const xmlChar*)systemId.c_str(),
+    publicId.empty() ? nullptr : (const xmlChar*)publicId.c_str(),
+    systemId.empty() ? nullptr : (const xmlChar*)systemId.c_str(),
     (const xmlChar*) content.c_str() );
   if (!entity)
     throw internal_error("Could not add entity declaration " + name);
 }
 
 int Document::process_xinclude(bool generate_xinclude_nodes)
+{
+  return process_xinclude(generate_xinclude_nodes, true);
+}
+
+int Document::process_xinclude(bool generate_xinclude_nodes, bool fixup_base_uris)
 {
   NodeMap node_map;
 
@@ -428,8 +433,13 @@ int Document::process_xinclude(bool generate_xinclude_nodes)
   find_wrappers(root, node_map);
 
   xmlResetLastError();
-  const int n_substitutions = xmlXIncludeProcessTreeFlags(root,
-    generate_xinclude_nodes ? 0 : XML_PARSE_NOXINCNODE);
+
+  int flags = 0;
+  if (!generate_xinclude_nodes)
+    flags |= XML_PARSE_NOXINCNODE;
+  if (!fixup_base_uris)
+    flags |= XML_PARSE_NOBASEFIX;
+  const int n_substitutions = xmlXIncludeProcessTreeFlags(root, flags);
 
   remove_found_wrappers(reinterpret_cast<xmlNode*>(impl_), node_map);
 
